;CODING STARTS HERE
;
;All Tutorials and Code are provided "as-is" for purposes of instruction and
;utility and may be used by anyone for any purpose entirely at their own risk.
;Please respect the intellectual rights of others.
;All material provided here is unsupported and without warranty of any kind.
;No responsibility will be taken for any direct or indirect consequences
;resulting from or associated with the use of these Tutorials or Code.
;*******************************************************************************
;			AfraLisp
;		http://www.afralisp.com
;	 	 afralisp@afralisp.com
;	   	 afralisp@mweb.com.na
;*******************************************************************************
;This application will extract attributes from a block and display them in a
;dialog box. The attributes will then be updated.
;
;Dependencies : Attab-vl.dcl and Attab-vl.dwg are
;required and must be within the AutoCAD search path.
;
;Usage : Open Attab-vl.dwg then load and run Attab-vl.lsp.
;*******************************************************************************

(prompt "\nATTAB-VL Loaded....Type ATTAB-VL to run.....")

(defun c:attab-vl (/)

;load visual lisp extensions
(vl-load-com)

;retrieve reference to the active document
(setq acadDocument (vla-get-activedocument (vlax-get-acad-object)))

;retrieve reference to the selection set collection
(setq ssets (vla-get-selectionsets acadDocument))

;check if the selection set exists - $Set
(if (vl-catch-all-error-p (vl-catch-all-apply 'vla-item (list ssets "$Set")))

  ;if it doesn't create a new one
  (setq newsset (vla-add ssets "$Set"))
  
     ;if it does exist
     (progn

	;delete it       
    	(vla-delete (vla-item ssets "$Set"))
       
	;then create a new one
  	(setq newsset (vla-add ssets "$Set"))
       
     );progn

);if

;create a single element array - integer
(setq filter_code (vlax-make-safearray vlax-vbinteger '(0 . 0)))

;create a single element array - variant
(setq filter_value (vlax-make-safearray vlax-vbvariant '(0 . 0)))

;filter for name - code 2
(vlax-safearray-fill filter_code '(2))

;filter for block name - attab-info
(vlax-safearray-fill filter_value '("attab-info"))

;filter the drawing for the block
(vla-select newsset acSelectionSetAll nil nil filter_code filter_value)

   ;if the block is found
   (if (>= (vla-get-count newsset) 1)

    ;display the dialog
    (ddisplay)

    ;if the block is not found
    (alert
      "\nIncorrect Drawing Sheet
      \n    Use Manual Edit"
    )

  );if

;finish clean	
(princ)

);defun

;;;**********************************************************

(defun ddisplay (/)

  	;load the dialog
  	(setq dcl_id (load_dialog "attab-vl.dcl"))

  	;check it exists
  	(if (not (new_dialog "attabvl" dcl_id))

    	(exit)

  	);if

	;retrieve the block reference
	(setq item (vla-item newsset 0))

	;retrieve the attributes
	(setq theatts (vla-getattributes item))

	;convert to a list
	(setq attlist (vlax-safearray->list (variant-value theatts)))

	;extract the attributes
	(mapcar 'set '(theattribute1 theattribute2 theattribute3 theattribute4 theattribute5) attlist)

	;extract the text strings from the attributes
	(setq eb1 (vla-get-textstring theattribute1)
	         eb2 (vla-get-textstring theattribute2)
	         eb3 (vla-get-textstring theattribute3)
	         eb4 (vla-get-textstring theattribute4)
	         eb5 (vla-get-textstring theattribute5)
	);setq				


  ;put the info into the dialog
  (set_tile "eb1" eb1)
  (set_tile "eb2" eb2)
  (set_tile "eb3" eb3)
  (set_tile "eb4" eb4)
  (set_tile "eb5" eb5)

  ;set the focus to the drawing number
  (mode_tile "eb1" 2)

  ;if cancel selected exit
  (action_tile
    "cancel"
    "(done_dialog) (setq userclick nil)"
  )

  ;if OK selected, retrieve the tile values
  (action_tile
    "accept"
    (strcat
      "(progn (setq eb1a (get_tile \"eb1\"))"
      "(setq eb2a (get_tile \"eb2\"))"
      "(setq eb3a (get_tile \"eb3\"))"
      "(setq eb4a (get_tile \"eb4\"))"
      "(setq eb5a (get_tile \"eb5\"))"
     " (done_dialog)(setq userclick T))"
    )
  )

  ;start the dialog
  (start_dialog)

  ;unload the dialog
  (unload_dialog dcl_id)


  ;if OK was selected
  (if userclick

	;do the following
	(progn

		;update the attribute textstrings
  	  	(vla-put-textstring theattribute1 eb1a)
		(vla-put-textstring theattribute2 eb2a)
		(vla-put-textstring theattribute3 eb3a)
		(vla-put-textstring theattribute4 eb4a)
		(vla-put-textstring theattribute5 eb5a)

		;update the block
		(vla-update newsset)

	  	;regen the drawing
          		(command "REGEN")

	);progn

   );if

);defun

;;;***********************************************************

;load clean
(princ)

;;;**********************************************************
;
;CODING ENDS HERE

